import { getEffectiveModelType } from "../../../core/checker.js";
import { createRekeyableMap } from "../../../utils/misc.js";
import { defineKit } from "../define-kit.js";
import { copyMap, decoratorApplication } from "../utils.js";
const spreadCache = new Map();
defineKit({
    model: {
        create(desc) {
            const properties = createRekeyableMap(Array.from(Object.entries(desc.properties)));
            const model = this.program.checker.createType({
                kind: "Model",
                name: desc.name ?? "",
                decorators: decoratorApplication(this, desc.decorators),
                properties: properties,
                node: undefined,
                derivedModels: desc.derivedModels ?? [],
                sourceModels: desc.sourceModels ?? [],
                indexer: desc.indexer,
            });
            this.program.checker.finishType(model);
            return model;
        },
        is(type) {
            return type.kind === "Model";
        },
        isExpresion(type) {
            return type.name === "";
        },
        getEffectiveModel(model, filter) {
            return getEffectiveModelType(this.program, model, filter);
        },
        getSpreadType(model) {
            if (spreadCache.has(model)) {
                return spreadCache.get(model);
            }
            if (!model.indexer) {
                return undefined;
            }
            if (model.indexer.key.name === "string") {
                const record = this.record.create(model.indexer.value);
                spreadCache.set(model, record);
                return record;
            }
            if (model.indexer.key.name === "integer") {
                const array = this.array.create(model.indexer.value);
                spreadCache.set(model, array);
                return array;
            }
            return model.indexer.value;
        },
        getProperties(model, options = {}) {
            // Add explicitly defined properties
            const properties = copyMap(model.properties);
            // Add discriminator property if it exists
            const discriminator = this.type.getDiscriminator(model);
            if (discriminator) {
                const discriminatorName = discriminator.propertyName;
                properties.set(discriminatorName, this.modelProperty.create({ name: discriminatorName, type: this.builtin.string }));
            }
            if (options.includeExtended) {
                let base = model.baseModel;
                while (base) {
                    for (const [key, value] of base.properties) {
                        if (!properties.has(key)) {
                            properties.set(key, value);
                        }
                    }
                    base = base.baseModel;
                }
            }
            // TODO: Add Spread?
            return properties;
        },
    },
});
//# sourceMappingURL=model.js.map